/********************************************************************
 * yalloc - string memory management library.  
 *   written for ModServ2x                     
 *   (C) 2005 Christian Jurk, commx@commx.ws   
 *   www.commx.ws, www.gamerirc.org 
 * 
 * You can use this functions in your private and/or commercial
 * programs. Read the GNU Public License (GPL) for more information.           
 *
 * Usage:
 * -------------------------
 *   yalloc_new(char *str);
 *   >> Request new entry.
 *   Example: yalloc_new("test");
 *   Returns: The memory-id to the entry or -1 when failed.
 *
 *   yalloc_replace(unsigned long id, char *newstr);
 *   >> Updates a entry with a new value.
 *   Example: yalloc_replace(1, "moo");
 *   Returns: 0 if success, -1  when failed.
 * 
 *   yalloc_get(unsigned long id);
 *   >> Gets the char-value.
 *   Example: yalloc_get(1);
 *   Returns: The value of the entry, NULL when failed.
 *
 *   yalloc_len(unsigned long id);
 *   >> Returns the length of a entry.
 *   Example: yalloc_len(1);
 *   Returns: The string length, or -1 when failed. 
 *
 *   yalloc_del(unsigned long id);
 *   >> Removes a entry.
 *   Example: yalloc_del(1);
 *   Returns: 0 if success, -1 when failed.
 *
 *   yalloc_stats();
 *   >> Prints a usage table.
 *   Example: yalloc_stats();
 *   Returns: The table.
 ********************************************************************/

#ifndef _USE_YALLOC
#define _USE_YALLOC

#include <stdio.h>
#include <stdlib.h>

struct yalloc
{
  char *string;
  unsigned int memid;
  struct yalloc *next;
};

struct yalloc *yptr = NULL;
unsigned long yc=0,fe=0;

int yalloc_new_(char *str)
{
  struct yalloc *y;
  y = (struct yalloc *)malloc(sizeof(struct yalloc));
  y->string = (char *)malloc(strlen(str)+1);
  if (y == NULL || y->string == NULL)
  {
    printf("yalloc_new: cannot request memory (malloc()/%d/1 byte safe)\r\n",strlen(str));
    return -1;
  }
  y->string[0] = 0x00;
  strncpy(y->string,str,strlen(str));
  y->string[strlen(str)]=0x00;
  y->next = yptr;
  yptr = y;
  yc++;
  y->memid = yc;
  return yc;
}

int yalloc_replace(unsigned long id, char *newstr)
{
  struct yalloc *y;
  for(y = yptr ; y != NULL ; y = y->next)
  {
    if (y->memid == id)
    {
      free(y->string);
      y->string = (char *)malloc(strlen(newstr)+1);
      if (y->string == NULL)
      {
        printf("yalloc_replace: cannot request memory (malloc()/%d/1 byte safe)\r\n",strlen(newstr));
      }
      strncpy(y->string,newstr,strlen(newstr));
      y->string[strlen(newstr)] = 0x00;
      return 0;
    }
  }
  return -1;
}

char *yalloc_get(unsigned long id)
{
  struct yalloc *y;
  for(y = yptr ; y != NULL ; y = y->next)
  {
    if (y->memid == id)
    {
      return y->string;
    }
  }
  return NULL;
}

int yalloc_len(unsigned long id)
{
  struct yalloc *y;
  for(y = yptr ; y != NULL ; y = y->next)
  {
    if (y->memid == id)
    {
      return (unsigned int)strlen(y->string);
    }
  }
  return -1;         
}

int yalloc_del(unsigned long id)
{
  struct yalloc *y, *x = NULL;
  for(y = yptr ; y != NULL ; y = y->next)
  {
    if (y->memid == id)
    {
      x->next = y->next;
      if (y == yptr)
        yptr = x;
        
      free(y->string);
      free(y);
      return 0;
    }
    x = y;
  }
  return -1;
}   

void yalloc_stats()
{
  // calculations: one character == one byte!
  struct yalloc *y; int s,x;
  printf("MemoryID\t\tMemory used\t\tValue/Length\r\n");
  printf("------------------------------------------------------------\r\n");
  for(y = yptr, s = 0, x = 0 ; y != NULL; y = y->next, s++)
  {
    printf("%u\t\t\t%d b\t\t\t%s (%d)\r\n", y->memid, strlen(y->string), y->string, strlen(y->string));
    x += strlen(y->string);
  }
  printf("------------------------------------------------------------\r\n");
  printf("%d Entry%s, %d byte%s.\r\n", s, (s == 1) ? "" : "s", x, (x == 1) ? "" : "s");
}

#endif /* _USE_YALLOC */
